c/*
c
c=======================================================
c       Plasma Surface Interaction (PSI) package
c  for simulation of plasma-wall sputtering processes.
c
c                 PSI Version 1.2   
c              Date: March 6, 2007
c
c
c Copyright 2007 by A.Yu. Pigarov and R.D. Smirnov
c
c
c                   Developers:
c          A.Yu. Pigarov and R.D. Smirnov
c
c
c         E-mails: apigarov@ucsd.edu
c                  rsmirnov@ucsd.edu
c
c    PSI package has been developed as part of
c   the Dust Transport in Tokamaks (DUSTT) code:
c         
c [1] A.Yu.Pigarov, S.I. Krasheninnikov, T.K. Soboleva,
c     and T.D. Rognlien "Dust-particle transport in
c     tokamak edge plasmas", Phys. Plasmas 12 (2005), 122508.
c [2] R.D. Smirnov, A.Yu. Pigarov, M. Rosenberg,
c     S.I. Krasheninnikov, and D.A. Mendis "Modelling of
c     dynamics and transport of carbon dust particles in 
c     tokamaks", Plasma Phys. Control. Fusion 49 (2007), 347-371.
c
c
c    The PSI package is distributed in the hope 
c that it will be useful to computational scientists in field
c of fusion plasma and other plasma related studies.
c
c    The PSI package is FREE SOFTWARE. 
c You can use, copy, and modify this software for any purpose
c and without fee provided that the above copyright
c notice appear in all copies.
c
c    The PSI package is distributed "AS IS", 
c i.e. without any warranty including all implied warranties
c of merchantability and fitness.
c   
c=======================================================
c
c*/
c
c      ===============================
c		Table Mask
c      ===============================
c       "h " ,"he" ,"li" ,"be" ,"b ",
c       "c " ,"n " ,"o " ,"f " ,"ne",
c       "na" ,"mg" ,"al" ,"si" ,"p ",
c       "s " ,"cl" ,"ar" ,"k " ,"ca",
c       "sc" ,"ti" ,"v " ,"cr" ,"mn",
c       "fe" ,"co" ,"ni" ,"cu" ,"zn",
c       "ga" ,"ge" ,"as" ,"se" ,"br",
c       "kr" ,"rb" ,"sr" ,"y " ,"zr",
c       "nb" ,"mo" ,"tc" ,"ru" ,"rh",
c       "pd" ,"ag" ,"cd" ,"in" ,"sn",
c       "sb" ,"te" ,"i " ,"xe" ,"cs",
c       "ba" ,"la" ,"ce" ,"pr" ,"nd",
c       "pm" ,"sm" ,"eu" ,"gd" ,"tb",
c       "dy" ,"ho" ,"er" ,"tm" ,"yb",
c       "lu" ,"hf" ,"ta" ,"w " ,"re",
c       "os" ,"ir" ,"pt" ,"au" ,"hg",
c       "tl" ,"pb" ,"bi" ,"po" ,"at",
c       "rn" ,"fr" ,"ra" ,"ac" ,"th",
c       "pa" ,"u "
c


      double precision function psimassa(iz)
      implicit none
      integer iz
c
c-----------------------------------------
c   atomic mass of chemical element iz
c-----------------------------------------
c
      real am(92)
      data am /
     ,    1.01,     4.00,    6.94,     9.01,   10.81,
     ,   12.01,    14.01,   16.00,    19.00,   20.18,
     ,   22.99,    24.31,   26.98,    28.09,   30.97,
     ,   32.07,    35.45,   39.95,    39.10,   40.08,
     ,   44.96,    47.88,   50.94,    52.00,   54.94,
     ,   55.85,    58.93,   58.69,    63.55,   65.39,
     ,   69.72,    72.59,   74.92,    78.96,   79.90,
     ,   83.80,    85.47,   87.62,    88.91,   91.22,
     ,   92.91,    95.94,   99.00,   101.07,  102.91,
     ,  106.42,   107.87,  112.41,   114.82,  118.71,
     ,  121.75,   127.60,  126.90,   131.29,  132.91,
     ,  137.33,   138.91,  140.12,   140.91,  144.24,
     ,  145.00,   150.36,  151.96,   157.25,  158.93,
     ,  162.50,   164.93,  167.26,   168.93,  173.04,
     ,  174.97,   178.49,  180.95,   183.85,  186.21,
     ,  190.20,   192.22,  195.08,   196.97,  200.59,
     ,  204.38,   207.20,  208.98,   210.00,  218.00,
     ,  222.00,   223.00,  226.00,   227.00,  232.00,
     ,  234.00,   236.00/

      psimassa=dble(am(iz))
      return

      end



      double precision function psi_izpot(iz)
      implicit none
      integer iz
c
c---------------------------------------------------------
c   first ionization potential of chemical element iz, eV
c---------------------------------------------------------
c
      real am(92)
      data am /
     ,  13.598,   24.587,   5.392,    9.322,   8.298,
     ,  11.26 ,   14.534,  13.618,   17.422,  21.564,
     ,   5.139,    7.646,   5.986,    8.151,  10.486,
     ,  10.36 ,   12.967,  15.759,    4.341,   6.113,
     ,   6.54 ,    6.82 ,   6.74 ,    6.766,   7.435,
     ,   7.87 ,    7.86 ,   7.635,    7.726,   9.394,
     ,   5.999,    7.899,   9.81 ,    9.752,  11.814,
     ,  13.999,    4.177,   5.695,    6.38 ,   7.586,
     ,   6.88 ,    7.099,   7.28 ,    7.37 ,   7.46 ,
     ,   8.34 ,    7.576,   8.993,    5.786,   7.344,
     ,   8.641,    9.009,  10.451,   12.13 ,   3.894,
     ,   5.212,    5.58 ,   5.54 ,    5.46 ,   5.53 ,
     ,   5.554,    5.64 ,   5.67 ,    6.15 ,   5.86 ,
     ,   5.94 ,    6.018,   6.101,    6.184,   6.254,
     ,   5.43 ,    6.65 ,   7.89 ,    7.98 ,   7.88 ,
     ,   8.7  ,    9.1  ,   9.   ,    9.225,  10.437,
     ,   6.108,    7.416,   7.289,    8.42 ,   0.0  ,
     ,  10.748,    4.08 ,   5.279,    5.17 ,   6.08 ,
     ,   5.89 ,    6.05 /

      psi_izpot=dble(am(iz))
      return

      end


      double precision function psiens(iz)
      implicit none
      integer iz
c
c
c-------------------------------------
c   matter density  (g/cm**3)
c c  c.kittel ''introduction to solid
c     state physics''
c    john wiley,new-york 1976
c -------------------------------------
c
      real ro(92)
      data ro /
     ,8.987e-5,1.785e-3,   0.534,   1.850,   2.320,
     ,   2.00 ,1.251e-3,1.429e-3,1.695e-3,9.000e-4,
     ,   0.970,   1.740,   2.702,   2.330,   2.200,
     ,   2.070,3.214e-3,1.784e-3,   0.860,   1.550,
     ,   2.500,   4.210,   6.110,   6.920,   7.200,
     ,   7.860,   8.900,   8.900,   8.920,   7.140,
     ,   5.910,   5.323,   5.727,   4.800,   3.120,
     ,3.487e-3,   1.532,   2.600,   4.470,   6.400,
     ,   8.400,  10.200,  11.500,  12.200,  12.500,
     ,  11.970,  10.500,   8.640,   7.310,   7.280,
     ,   6.684,   6.240,   4.930,5.890e-3,   1.900,
     ,   3.760,   6.162,   6.768,   6.770,   7.007,
     ,   7.000,   7.536,   5.300,   7.900,   8.300,
     ,   8.500,   8.800,   9.160,   9.300,   7.000,
     ,   0.849,  13.310,  16.600,  19.300,  20.530,
     ,  22.480,  22.420,  21.450,  19.300,  13.546,
     ,  11.850,  11.344,   9.800,   9.320,   5.000,
     ,9.730e-3,   2.400,   5.000,  10.100,  11.200,
     ,  15.400,  18.700 /

      psiens=dble(ro(iz))
      return

      end



      double precision function psiwsub(iz)
      implicit none
      integer iz
c
c-----------------------------------------------------------------------
c   sublimation energy of iz-matter  (ev) (at 298K ?)
c	 i p p j - a m - 32
c-----------------------------------------------------------------------
c
      real eb(92)
      data eb /
     , 0.01, 0.01, 1.63, 3.32, 5.75,
     , 7.37, 4.92, 2.60, 0.84, 0.02,
     , 1.11, 1.51, 3.39, 4.63, 3.43,
     , 2.85, 1.40, 0.08, 0.93, 1.84,
     , 3.90, 4.85, 5.31, 4.10, 2.92,
     , 4.28, 4.39, 4.44, 3.49, 1.35,
     , 2.81, 3.85, 2.96, 2.25, 1.22,
     , 0.12, 0.85, 1.72, 4.37, 6.25,
     , 7.57, 6.81, 6.85, 6.74, 5.75,
     , 3.89, 2.95, 1.16, 2.52, 3.14,
     , 2.75, 2.23, 1.11, 0.16, 0.80,
     , 1.90, 4.47, 4.32, 3.70, 3.40,
     , 3.03, 2.14, 1.86, 4.14, 4.05,
     , 3.04, 3.14, 3.29, 2.42, 1.60,
     , 4.43, 6.44, 8.10, 8.81, 8.03,
     , 8.17, 6.94, 5.84, 3.81, 0.67,
     , 1.88, 2.03, 2.18, 1.50, 1.02,
     , 0.20, 1.02, 1.66, 4.25,  6.2,
     , 5.50, 5.55/

      psiwsub=dble(eb(iz))
      return

      end



      double precision function psicel(iz)
      implicit none
      integer iz 
c
c--------------------------
c  electron energy loss
c  correction factors
c--------------------------
c
       real d(92)
       data d /
     , 0.9341,0.6693,0.6654,0.9712,
     ,  1.007, 1.024, 1.111,0.9699,
     , 0.7357,0.6842,0.8769,  1.29,
     ,  1.395, 1.378, 1.063, 1.123,
     ,  1.632, 1.839, 1.642, 1.749,
     ,  1.638, 1.523, 1.396, 1.236,
     ,  1.072, 1.083,0.9624, 1.085,
     ,  1.125, 1.277, 1.525, 1.675,
     ,  1.601, 1.762, 1.679, 1.914,
     ,  1.696, 1.884,   1.9, 1.993,
     ,  2.039, 1.894, 2.001, 1.795,
     ,  1.738, 1.534, 1.644, 1.698,
     ,  1.816, 1.866, 2.181, 2.027,
     ,   2.24, 2.384, 2.108, 2.283,
     ,  2.321, 2.159,   2.1, 2.042,
     ,  1.986, 1.932, 1.879, 1.931,
     ,  1.779, 1.578, 1.492, 1.448,
     ,  1.406, 1.365, 1.394, 1.431,
     ,  1.348,   1.3, 1.477, 1.439,
     ,  1.403, 1.269, 1.376,  1.22,
     ,  1.336, 1.504, 1.683, 1.739,
     ,  1.751, 1.744, 1.959, 2.115,
     ,  2.167,  2.17, 2.084,  2.05/

      psicel=dble(d(iz))
      return

      end


      double precision function psia0(iz)
      implicit none
      integer iz
c
c---------------------------------------------------
c   average lattice constant of random target (A)
c      IPPJ-AM-26
c---------------------------------------------------
c
      real ao(92)
      data ao /
     ,  2.66, 3.34,  2.8, 2.01, 1.98,
     ,  2.07, 2.83, 2.65, 2.53, 2.78,
     ,  3.41, 2.86, 2.56, 2.72, 3.05,
     ,  2.96, 3.05, 3.35, 4.23, 3.52,
     ,  2.93, 2.61, 2.41, 2.29, 2.31,
     ,  2.28, 2.23, 2.22, 2.28, 2.48,
     ,   2.7, 2.83,  2.8, 3.02, 3.19,
     ,  3.57, 4.53, 3.83, 3.22, 2.86,
     ,  2.64,  2.5, 2.43,  2.4,  2.4,
     ,  2.46, 2.58, 2.79, 2.97,   3.,
     ,  3.13, 3.24,  3.5, 3.87, 4.89,
     ,  4.03, 3.35, 3.26, 3.26, 3.25,
     ,    3., 3.22, 3.18, 3.17, 3.14,
     ,  3.14, 3.12,  3.1, 3.12, 3.46,
     ,   3.1, 2.83, 2.63, 2.51, 2.46,
     ,  2.41, 2.42, 2.48, 2.57, 2.91,
     ,  3.06, 3.12, 3.29, 3.34,   3.,
     ,  4.38,   4., 4.23, 3.35, 3.21,
     ,  2.93, 2.75 /

      psia0=dble(ao(iz))
      return

      end


      double precision function psi_Ef(iz)
      implicit none
      integer iz
c
c----------------------------------
c    Fermi energy (eV)
c----------------------------------
c
      real eb(92)
      data eb/
     , -1.00,-1.00, 4.70,14.30, 9.30,
     , 13.41,-1.00,-1.00,-1.00,-1.00,
     ,  3.10, 7.10,11.60, 7.0 , 7.0 ,
     ,	7.0 ,-1.00,-1.00, 2.00, 4.70,
     ,	6.00, 8.50,10.00, 6.70,10.60,
     , 11.20,11.60,11.60, 7.00, 9.50,
     ,	7.30, 7.0 , 7.0 , 7.0 , 7.0 ,
     , -1.00, 1.70, 4.00, 5.60, 7.00,
     ,	5.20, 5.80, 6.20, 6.30, 6.40,
     , 28.00, 5.50, 7.40, 8.60,10.70,
     , 10.90, 9.0 , 9.0 ,-1.00, 1.50,
     ,  2.50, 5.20, 5.50, 5.50, 5.50,
     ,	5.50, 5.60, 4.40, 5.50, 5.80,
     ,	5.80, 5.80, 5.90, 5.90, 4.90,
     ,	1.20, 7.30, 8.40, 9.10, 9.40,
     ,	9.90, 9.80, 5.90, 5.50, 6.80,
     ,	6.50, 9.40, 8.50,10.4 , 4.00,
     , -1.00, 1.30, 3.20, 2.7 , 3.40,
     ,	3.30, 7.60/

      psi_Ef=dble(eb(iz))
      return

      end


      double precision function psi_dens(iz)
      implicit none
      integer iz
c
c-------------------------------------
c  matter density  (A**-3)
c
c  c.kittel ''introduction to solid
c     state physics''
c    john wiley,new-york 1976
c-------------------------------------
c
      real d(92)
      data d/
     , 5.370e-3,2.687e-3,   4.636,  12.368,  12.929,
     ,   11.286,5.379e-3,5.381e-3,5.375e-3,2.687e-3,
     ,    2.542,   4.312,   6.033,   4.998,   4.279,
     ,    3.890,5.462e-3,2.690e-3,   1.325,   2.330,
     ,    3.350,   5.660,   7.226,   8.018,   7.896,
     ,    8.479,   9.099,   9.133,   8.458,   6.581,
     ,    2.980,   4.418,   4.605,   3.662,   2.352,
     , 2.510e-3,   1.080,   1.788,   3.029,   4.227,
     ,    5.459,   6.405,   6.999,   7.273,   7.318,
     ,    6.778,   5.865,   4.631,   3.830,   3.995,
     ,    3.308,   2.946,   2.341,2.703e-3,   0.861,
     ,    1.649,   2.673,   2.910,   2.895,   2.927,
     ,    3.000,   3.020,   2.101,   3.027,   3.147,
     ,    3.151,   3.215,   3.299,   3.317,   2.437,
     ,    0.292,   4.493,   5.527,   6.325,   6.643,
     ,    7.121,   7.028,   6.624,   5.904,   4.069,
     ,    3.493,   3.299,   2.825,   2.674,   2.000,
     , 2.641e-3,   0.648,   1.333,   3.000,   2.908,
     ,    4.000,   4.773/

      psi_dens=dble(d(iz))*1.d-2
      return

      end


      integer function psimatype(iz)
      implicit none
      integer iz
c
c---------------------------------------
c	  matter type :
c
c    j= -1  - gas
c	 0  - not constuctional
c	 1  - constructional metal
c	 2  - shielding layer
c---------------------------------------
c
      integer id(92)
      data id/ 
     , -1    , -1    ,  2    ,  2    ,  2    ,
     ,  1    , -1    , -1    , -1    , -1    ,
     ,  2    ,  0    ,  1    ,  1    ,  0    ,
     ,  0    , -1    , -1    ,  0    ,  0    ,
     ,  0    ,  1    ,  1    ,  1    ,  0    ,
     ,  1    ,  1    ,  1    ,  1    ,  1    ,
     ,  0    ,  1    ,  0    ,  0    ,  0    ,
     , -1    ,  0    ,  0    ,  0    ,  1    ,
     ,  1    ,  1    ,  1    ,  0    ,  0    ,
     ,  0    ,  2    ,  0    ,  0    ,  0    ,
     ,  0    ,  2    ,  0    , -1    ,  0    ,
     ,  2    ,  0    ,  0    ,  0    ,  0    ,
     ,  0    ,  0    ,  0    ,  0    ,  0    ,
     ,  0    ,  0    ,  0    ,  0    ,  0    ,
     ,  0    ,  0    ,  1    ,  1    ,  0    ,
     ,  0    ,  0    ,  2    ,  2    ,  0    ,
     ,  0    ,  2    ,  2    ,  0    ,  0    ,
     ,  0    ,  0    ,  0    ,  0    ,  2    ,
     ,  0    ,  2    /

      psimatype=id(iz)
      return

      end


      double precision function psi_Aew(iz)
      implicit none
      integer iz
c
c---------------------------------------------
c      electron extraction potential
c            (work function)
c	  out from iz-matter  (eV)
c---------------------------------------------
c
      real ed(92)
      data ed/
     , -1.00,-1.00, 2.38, 3.92, 4.50,
     ,	4.71,-1.00,-1.00,-1.00,-1.00,
     ,	2.35, 3.64, 4.25, 4.80, 2.0 ,
     ,	2.0 ,-1.00,-1.00, 2.22, 2.60,
     ,	3.30, 3.95, 4.12, 4.58, 3.83,
     ,	4.31, 4.41, 4.50, 4.40, 4.24,
     ,  3.96, 4.76, 5.11, 4.72, 4.0 ,
     , -1.00, 2.16, 2.35, 3.30, 3.90,
     ,	3.99, 4.30, 4.44, 4.60, 4.75,
     ,	4.80, 4.30, 4.10, 3.80, 4.38,
     ,	4.08, 4.73, 6.8 ,-1.00, 1.81,
     ,	2.49, 3.30, 2.70, 2.7 , 3.2 ,
     ,	3.07, 2.7 , 2.5 , 3.1 , 3.15,
     ,	3.25, 3.22, 3.25, 3.1 , 2.6 ,
     ,	3.14, 3.53, 4.12, 4.54, 5.00,
     ,	4.70, 4.70, 5.32, 4.30, 4.52,
     ,	3.70, 4.00, 4.40, 4.7 , 4.0 ,
     , -1.00, 1.60, 3.20, 2.7 , 3.30,
     ,  3.30, 3.30/

      psi_Aew=dble(ed(iz))
      return

      end


      double precision function psi_TDebye(iz)
      implicit none
      integer iz
c
c-----------------------------
c   Debye temperature,K
c-----------------------------
c
      real Td(92)
      data Td/
     ,  110. , 0.0   , 448.  , 1031. , 1250.,
     ,  1860., 79.   , 46.   , 0.0   , 63.  ,
     ,  155. , 330.  , 394.  , 625.  , 0.0  ,
     ,  0.0  , 0.0   , 85.   , 100.  , 230. ,
     ,  476. , 380.  , 390.  , 424.  , 383. ,
     ,  373. , 386.  , 375.  , 315.  , 234. ,
     ,  240. , 360.  , 285.  , 150.  , 0.0  ,
     ,  73.  , 59.   , 148.  , 214.  , 250. ,
     ,  260. , 377.  , 422.  , 415.  , 350. ,
     ,  275. , 215.  , 120.  , 129.  , 170. ,
     ,  200. , 135.  , 0.0   , 0.0   , 43.  ,
     ,  116. , 135.  , 138.  , 138.  , 148. ,
     ,  0.0  , 148.  , 107.  , 176.  , 188. ,
     ,  186. , 191.  , 195.  , 200.  , 118. ,
     ,  207. , 213.  , 225.  , 312.  , 275. ,
     ,  400. , 228.  , 230.  , 170.  , 100. ,
     ,  96.  , 88.   , 120.  , 0.0   , 0.0  ,
     ,  0.0  , 77.   , 85.   , 118.  , 100. ,
     ,  262. , 300.  /

      psi_TDebye=dble(Td(iz))
      return

      end


      double precision function psi_Tmelt(iz)
      implicit none
      integer iz
c---------------------------------------
c  melting (or sublimation) temperature
c     gradus C
c---------------------------------------
      real Td(92)
      data Td/
     , -259.19 , -271.4  ,  180.54 , 876.85  , 1476.85 ,
     , 3806.43 , -210.012, -218.79 , -219.6  , -248.52 ,
     ,   97.9  ,  650.0  ,  660.24 , 1415.0  ,   44.1  ,
     ,  119.3  , -101.03 , -189.30 ,   63.5  ,  842.0  ,
     , 1541.0  , 1608.0  , 1917.0  , 1877.0  , 1245.0  ,
     , 926.85  , 1492.0  , 1455.0  , 1083.0  ,  419.5  ,
     ,   29.78 ,  237.2  ,  615.0  ,  217.0  ,    7.25 ,
     , -157.37 ,   39.49 ,  768.0  , 1528.0  , 1852.0  ,
     , 2469.0  , 1876.85 , 2200.0  , 2250.0  , 1963.0  ,
     , 1554.0  ,  960.5  ,  320.9  ,  156.4  ,  231.97 ,
     ,  630.5  ,  449.8  ,  113.0  , -111.85 ,   28.5  ,
     ,  727.0  ,  920.0  ,  799.0  ,  932.0  , 1024.0  ,
     , 1100.0  , 1072.0  ,  822.0  , 1313.0  , 1356.0  ,
     , 1409.0  , 1474.0  , 1525.0  , 1545.0  ,  824.0  ,
     , 1663.0  , 2230.0  , 2996.0  , 2176.85 , 3180.0  ,
     , 3045.0  , 2447.0  , 1772.0  , 1063.0  ,  -38.89 ,
     ,  303.5  ,  327.44 ,  217.4  ,  254.0  ,  302.0  ,
     ,  -71.0  ,   26.84 ,  700.0  , 1050.0  , 1750.0  ,
     , 1572.0  , 1134.0  /

      psi_Tmelt=dble(Td(iz))
      return

      end


      double precision function psi_Hmelt(iz)
      implicit none
      integer iz
c---------------------------
c  melting enthalpy
c     kJ/mol
c---------------------------
      real Td(92)
      data Td/
     , 0.117  , 0.007  ,  3.000 , 15.508 , 48.927 ,
     , 711.19 , 0.7207 ,  0.4459,  0.51  ,  0.33  ,
     ,  2.60  ,  8.56  ,  10.8  ,  49.8  ,  0.66  ,
     ,  1.72  ,  6.61  ,  1.190 ,  2.33  ,  8.66  ,
     ,  14.2  ,  15.1  ,  17.6  ,  16.3  ,  12.1  ,
     ,  9.362 ,  16.3  ,  17.6  ,  13.0  ,  7.20  ,
     ,  5.59  ,  37.6  ,  31.8  ,  6.70  ,  10.57 ,
     ,  1.64  ,  2.19  ,  8.00  ,  11.0  ,  14.6  ,
     ,  28.0  , 41.568 ,  24.0  ,  24.3  ,  21.8  ,
     ,  17.0  ,  11.3  ,  6.20  ,  3.34  ,  7.20  ,
     ,  20.1  ,  17.5  ,  15.77 ,  2.29  ,  2.10  ,
     ,  7.53  ,  6.30  ,  5.20  ,  6.90  ,  10.68 ,
     ,  10.5  ,  8.60  ,  9.2   ,  10.0  ,  10.8  ,
     ,  10.9  ,  12.4  ,  19.9  ,  16.9  ,  7.70  ,
     ,  18.8  ,  23.9  ,  35.0  , 46.897 ,  33.0  ,
     ,  31.8  ,  26.3  ,  20.0  ,  12.6  ,  2.30  ,
     ,  4.20  ,  4.77  ,  11.0  ,  12.6  ,  8.80  ,
     ,  2.89  ,  2.10  ,  9.20  ,  12.0  ,  16.7  ,
     ,  16.7  ,  9.20  /
			
      psi_Hmelt=dble(Td(iz))
      return

      end


      double precision function psi_Hmelt_ev(iz)
      implicit none
      integer iz
c---------------------------
c  melting enthalpy
c     eV
c---------------------------
      double precision psi_Hmelt

      double precision f
      real mol1,gkjtoev

      data mol1    /1.66053873  /    !amu*10^24 or 10^24/mole
      data gkjtoev /0.6241509745/    !*10^22 eV

      f=psi_Hmelt(iz)
      f=f*(mol1*gkjtoev*1.d-2) !kJ/mol->eV

      psi_Hmelt_ev=f
      return

      end


      double precision function psi_Tsub(iz)
      implicit none
      integer iz
c--------------------------------------
c   boiling or sublimation temperature
c      gradus C
c--------------------------------------
      real Td(92)
      data Td/
     , -252.77 , -268.934, 1334.63, 2465.84 , 3862.38 ,
     , 3806.43 , -195.81 , -182.97, -188.13 , -245.93 ,
     ,  886.0  , 1107.0  , 2520.0 , 3300.0  ,  257.0  ,
     ,  444.6  ,  -34.10 , -185.9 ,  761.0  , 1484.0  ,
     , 2831.0  , 3287.0  , 3392.0 , 2672.0  , 2080.0  ,
     , 2860.195, 2957.0  , 2800.0 , 2543.0  ,  906.2  ,
     , 2300.0  , 2847.0  ,  615.0 ,  685.0  ,   59.2  ,
     , -153.22 ,  686.04 , 1384.0 , 3337.0  , 4340.0  ,
     , 4800.0  , 4678.819, 4260.0 , 4050.0  , 3680.0  ,
     , 3000.0  , 2167.0  ,  766.5 , 2024.0  , 2450.0  ,
     , 1634.0  ,  990.0  ,  184.35, -107.96 ,  672.0  ,
     , 1640.0  , 3454.0  , 3340.0 , 3360.0  , 3080.0  ,
     , 2600.0  , 1778.0  , 1597.0 , 3280.0  , 3200.0  ,
     , 2562.0  , 2695.0  , 2690.0 , 1840.0  , 1211.0  ,
     , 3395.0  , 4900.0  , 5350.0 , 5651.13 , 5627.0  ,
     , 5027.0  , 4380.0  , 3827.0 , 2877.0  ,  356.66 ,
     , 1457.0  , 1745.0  , 1552.0 ,  962.0  ,  334.0  ,
     ,  -61.9  ,  677.0  , 1320.0 , 3297.0  , 4790.0  ,
     , 3780.0  , 4030.0 /

      psi_Tsub=dble(Td(iz))
      return

      end


      double precision function psi_Hsub(iz)
      implicit none
      integer iz
c--------------------------------------
c  boiling or sublimation
c     enthalpy kJ/mol
c--------------------------------------
      real Td(92)
      data Td/
     ,   0.916,   0.0837, 157.7  , 320.0  , 555.0  ,
     , 711.19 ,   5.59  ,   6.833,   6.54 ,   1.79 ,
     ,  90.1  , 128.0   , 293.0  , 356.0  ,  13.1  ,
     ,   9.20 ,  20.41  ,   6.50 ,  76.6  , 152.0  ,
     , 315.0  , 410.0   , 445.0  , 338.0  , 227.0  ,
     , 413.1  , 376.0   , 370.0  , 302.0  , 115.3  ,
     , 256.0  , 334.0   ,  31.8  ,  29.0  ,  29.50 ,
     ,   9.046,  70.0   , 134.0  , 362.0  , 558.0  ,
     , 662.0  , 657.37  , 593.0  , 602.0  , 496.0  ,
     , 353.0  , 251.0   ,  99.6  , 228.0  , 296.1  ,
     , 124.4  ,  51.00  ,  41.8  ,  12.63 ,  67.0  ,
     , 139.0  , 413.0   , 409.0  , 297.0  , 255.0  ,
     , 111.0  , 166.0   , 147.0  , 360.0  , 331.0  ,
     , 228.0  , 240.0   , 261.0  , 191.0  , 130.0  ,
     , 356.0  , 569.0   , 745.0  , 849.8  , 716.0  ,
     , 748.0  , 613.0   , 511.0  , 331.0  ,  59.23 ,
     , 162.4  , 178.0   , 177.0  ,  59.0  ,  90.40 ,
     ,  16.8  ,  64.0   , 132.0  , 378.0  , 540.0  ,
     , 481.5  , 494.0   /

      psi_Hsub=dble(Td(iz))
      return

      end


      double precision function psi_eBB(iz)
      implicit none
      integer iz
c--------------------------------------
c   Black Body emissivity factor
c--------------------------------------
      real eps(92)
      data eps/
     , 0.9 , 0.9 , 0.1 , 0.15, 0.15,
     , 0.75, 0.9 , 0.9 , 0.9 , 0.9 ,
     , 0.15, 0.15, 0.1 , 0.15, 0.2 ,
     , 0.2 , 0.9 , 0.9 , 0.1 , 0.1 ,
     , 0.2 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.35, 0.1 , 0.1 , 0.02, 0.1 ,
     , 0.1 , 0.1 , 0.2 , 0.2 , 0.9 ,
     , 0.9 , 0.2 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.2 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.9 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.4 , 0.1 ,
     , 0.1 , 0.1 , 0.02, 0.02, 0.02,
     , 0.1 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 , 0.1 , 0.1 , 0.1 ,
     , 0.1 , 0.1 /

      psi_eBB=dble(eps(iz))
      return

      end


      integer function psi_melt(iz)
      implicit none
      integer iz
c--------------------------------------
c   wether the matter has liquid phase
c   "1" - yes, "0" - no
c--------------------------------------
      integer ml(92)
      data ml/
     , 0 , 1 , 1 , 1 , 1 ,
     , 0 , 0 , 0 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 0 ,
     , 0 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 0 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 , 1 , 1 , 1 ,
     , 1 , 1 /

      psi_melt=ml(iz)
      return

      end


      real*8 function psi_cncspt(iz)
      implicit none
      integer iz
c---------------------------------------------
c   correction factor for chemical sputtering
c   taking into account molecule sputtering
c---------------------------------------------
      real cn(92)
      data cn/
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.4 ,
     , 1.4 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 /

      psi_cncspt=cn(iz)
      return

      end


      real*8 function psi_cnres(iz)
      implicit none
      integer iz
c---------------------------------------------
c   correction factor for RES
c   taking into account molecule sputtering
c---------------------------------------------
      real cn(92)
      data cn/
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.8 ,
     , 1.8 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 , 1.1 , 1.1 , 1.1 ,
     , 1.1 , 1.1 /

      psi_cnres=cn(iz)
      return

      end
